
# This script uses Brian2 to produce network activity that mimics cypin 
# overexpression in vitro by modulating the AMPA conductance, connection 
# density, and external noise for neurons in the in silico network.

# Briefly, this code outputs which neurons are spiking and at what time they 
# are spiking. This network activity was later analysed in Matlab 
# (cypinOverexpressionAnalysis.m) to produce Figures 10 and 11 in 
# Rodriguez, et al., Cytosolic PSD-95 Interactor Regulates AMPA Receptor 
# Signaling Independent of PSD-95 Binding. Network Neurosci, 2020.

# This code was adapted from Masquelier and Deco, PLOS ONE, 2013 by Erin D.
# Anderson and Nicholas F. Vigilante and can be accessed at 
# https://www.seas.upenn.edu/~molneuro/

#%% Import packages
from brian2 import *
from itertools import combinations
import networkx as nx
import math
import numpy as np
import numpy.matlib as matlib
from networkx import NetworkXError
import random
import scipy.io as sio
import gc

#%% Defining baseline network model parameters (almost all of them have been extracted from Masquelier et al. 2013 PlOS ONE)
    
# Simulation Parameters
simtime = 300*second           # Simulation time for simulation steps
pre_simtime = 10*second        # Simulation time for stabilization step
defaultclock.dt = 0.1*ms       # simulation step length [ms]

condDelay = 3*ms              # experimental estimation of conditional delay of firing

# Pyramidal (excitatory) cells 
Cm_e        = 0.5*nF     # [nF] total capacitance 
gl_e        = 25.0*nS    # [ns] total leak conductance 
El_e        = -70.0*mV   # [mV] leak reversal potential 
Vth_e       = -50.0*mV   # [mV] threshold potential
Vr_e   = -60.0*mV        # [mV] reset potential 
tr_e   = 2.0*ms          # [ms] refractory time 

# Interneuron (inhibitory) cells 
Cm_i        = 0.2*nF     # [nF] total capacitance 
gl_i        = 20.0*nS    # [ns] total leak conductance 
El_i        = -70.0*mV   # [mV] leak reversal potential 
Vth_i       = -50.0*mV   # [mV] threshold potential 
Vr_i   = -60.0*mV        # [mV] reset potential 
tr_i   = 1.0*ms          # [ms] refractory time


# Synaptic weights
w_ee = 8.75
w_ei = w_ee
w_ie = 1
w_ii = 1

# Noise parameters
sigma_noise  = 6.5*mV        # gaussian white noise (Langevin equation)

# AMPA receptor
E_ampa   = 0.0*mV             # synaptic reversal potential (mV)
tau_ampa = 5.0*ms             # Glutamatergic synaptic time constant (AMPA)
g_ampa_e = 0.104*nS
g_ampa_i = 0.081*nS

# GABA receptor
E_gaba     = -70.0*mV        # inhibitory synaptic reversal potential (mV)
tau_gaba   = 10.0*ms         # GABAergic synaptic time constant
g_gaba_e   = 1.250*nS
g_gaba_i   = 0.973*nS

# NMDA receptor 
E_nmda     = 0.0*mV          # synaptic reversal potential (mV)
tau_s_nmda = 100.0*ms        # decay time of NMDA currents 
tau_x_nmda = 2.0*ms          # controls the rise time of NMDAR channels (ms)
a_nmda     = 0.5*kHz         # controls the saturation properties of NMDAR channels (kHz)
g_nmda_e = 0.327*nS
g_nmda_i = 0.258*nS

# STDP parameter for E -> E connections               
dApre = 1e-2
dApost = - 0.55*dApre
tau_post=34.0*ms             #(source: Bi & Poo 2001)
tau_pre=17.0*ms              #(source: Bi & Poo 2001) 
wmax = 2*w_ee

# STP parameters (Masquelier and Deco, 2013)
tau_f = 1600*ms              # Facilitation time scale
tau_d = 800*ms               # Depression time scale
U     = 0.025                

# Adaptation parameters (Masquelier and Deco, 2013); Modeling calcium-activated potassium current for spike-frequency adaptation (Wang et al 1999)
a_Ca = 0.0145
g_AHP = 10*nS
tau_Ca = 8000*ms
E_K = -80*mV

# External Input parameters
mu = 5
sigma = 1
I_on = 200*pA

#%% Defining neurodynamics equations

# Define equations describing the dynamics of excitatory neurons
exc_eqs = '''
            removed : boolean
            inputCurrent : 1
            dv/dt=int(not removed)*((inputCurrent*sigma_noise*xi*(2*gl_e/Cm_e)**.5) + (-gl_e*(v-El_e) - g_ampa_e*(v-E_ampa)*s_ampa_in - s_nmda_in*g_nmda_e*(v-E_nmda)/(1+exp(-0.062*v/volt)/3.57) - g_gaba_e*(v-E_gaba)*s_gaba - g_AHP*Ca*(v-E_K) + I)/Cm_e): volt (unless refractory)
            ds_ampa/dt = -s_ampa/tau_ampa : 1
            ds_nmda/dt = -(s_nmda/tau_s_nmda) + a_nmda*x*(1-s_nmda) : 1
            ds_gaba/dt = -s_gaba/(tau_gaba) : 1
            dx/dt = -x/tau_x_nmda : 1
            dCa/dt = -Ca/tau_Ca : 1
            I : amp
            w : 1
            s_ampa_in = s_ampa_in1 + s_ampa_in2 + s_ampa_in3: 1
            s_nmda_in = s_nmda_in1 + s_nmda_in2 + s_nmda_in3: 1
            s_ampa_in1 : 1
            s_ampa_in2 : 1
            s_ampa_in3 : 1
            s_nmda_in1 : 1
            s_nmda_in2 : 1
            s_nmda_in3 : 1
            tmp_nmda : 1
            tmp_ampa : 1
         '''

# Define equations describing the dynamics of inhibitory neurons
inh_eqs = '''
            removed : boolean
            dv/dt=int(not removed)*((sigma_noise*xi*(2*gl_i/Cm_i)**.5) + (-gl_i*(v-El_i) - g_ampa_i*(v-E_ampa)*s_ampa_in - s_nmda_in*g_nmda_i*(v-E_nmda)/(1+exp(-0.062*v/volt)/3.57) - g_gaba_i*(v-E_gaba)*s_gaba - g_AHP*Ca*(v-E_K) + I)/Cm_i): volt (unless refractory)
            ds_ampa/dt = -s_ampa/tau_ampa : 1
            ds_nmda/dt = -(s_nmda/tau_s_nmda) + a_nmda*x*(1-s_nmda) : 1
            ds_gaba/dt = -s_gaba/tau_gaba : 1
            dx/dt = -x/tau_x_nmda : 1
            dCa/dt = -Ca/tau_Ca : 1
            I : amp
            w : 1
            s_ampa_in = s_ampa_in1 + s_ampa_in2 + s_ampa_in3: 1
            s_nmda_in = s_nmda_in1 + s_nmda_in2 + s_nmda_in3: 1
            s_ampa_in1 : 1
            s_ampa_in2 : 1
            s_ampa_in3 : 1
            s_nmda_in1 : 1
            s_nmda_in2 : 1
            s_nmda_in3 : 1
         '''     

#%% Iterate through parameters
g_ampa_e_baseline = g_ampa_e

nReplicates = 5
multipliers = matlib.repmat(np.array([[1,1,2],[1,0.75,2],[1,2,2]]),1,nReplicates) # AMPA, connection density, I_pre
nConditions = np.size(multipliers,1)/nReplicates

for n in range(np.size(multipliers,1)): # Iterate through all the experimental conditions

    start_scope()
    
    g_ampa_e = g_ampa_e_baseline * multipliers[0,n]
    # I_on = I_on_baseline * multipliers[n]
    
    print(f">>> Starting sim {n+1} of {np.size(multipliers,1)} with AMPA x{multipliers[0,n]}, connection density x{multipliers[1,n]}, and presynaptic current x{multipliers[2,n]}.")
    
    # Network structure
    N = 500 # Total number of neurons
    NE = math.floor(0.8*N)        # Number of Excitatory neurons
    NI = N - NE                   # Number of Inhibitory neurons
    
    #%% Define neuronal groups
    # Excitatory neuronal group
    Pe = NeuronGroup(NE, model=exc_eqs, threshold="v>Vth_e",
                    reset="v=Vr_e", refractory=tr_e, 
                    method='euler')
    
    
    # Inhibitory neuronal group
    Pi = NeuronGroup(NI, model=inh_eqs, threshold="v>Vth_i",
                    reset="v=Vr_i", refractory=tr_i, 
                    method='euler')
    
    #%% Define connections
    # Static Excitatory Connections (autapses)
    self_exc = Synapses(Pe, Pe, model = '''
                                        du_f/dt = (U-u_f)/tau_f : 1 (event-driven)
                                        dx_d/dt = (1-x_d)/tau_d : 1 (event-driven)
                                        s_nmda_in1_post = int(not removed) * tmp_nmda_pre : 1 (summed)
                                        s_ampa_in1_post = int(not removed) * tmp_ampa_pre : 1 (summed)
                                        ''',
                         delay=condDelay,
                         on_pre='''
                                u_f += int(not removed) * U * (1 - u_f)
                                r_S = int(not removed) * u_f * x_d
                                x_d -= int(not removed) * r_S
                                s_ampa_post += int(not removed) * w
                                x_post += int(not removed) * w
                                tmp_ampa = int(not removed) * clip(s_ampa*r_S*w_ee, 0, 100*r_S*w_ee)
                                tmp_nmda = int(not removed) * clip(s_nmda*r_S*w_ee, 0, 100*r_S*w_ee)
                                Ca += int(not removed) * a_Ca
                                ''')
    
    self_exc.connect(condition='i==j', p=1) 
    self_exc.w = 1
    self_exc.u_f = U  #all synapses have fully-replenished neurotransmitter resources
    self_exc.x_d = 1  #all synapses have fully-replenished neurotransmitter resources
    
    
    # Static Inhibitory Connections (autapses)
    # [Autapses use short term plasticity (STP)]
    self_inh = Synapses(Pi, Pi, model = '''
                                        du_f/dt = (U-u_f)/tau_f : 1 (event-driven)
                                        dx_d/dt = (1-x_d)/tau_d : 1 (event-driven)
                                        ''', 
                         delay=condDelay,
                         on_pre='''
                                u_f += int(not removed) * U * (1 - u_f)
                                r_S = int(not removed) * u_f * x_d
                                x_d -= int(not removed) * r_S
                                s_gaba_post += int(not removed) * r_S * w
                                Ca += int(not removed) * a_Ca
                                ''')
    
    self_inh.connect(condition='i==j', p=1) 
    self_inh.w = 1
    self_inh.u_f = U
    self_inh.x_d = 1
    
    # Remaining connections
    
    # Excitatory - Excitatory connections (with additional STDP)
    con_ee = Synapses(Pe, Pe, model = '''
                                        w_STDP : 1
                                        dApre/dt=-Apre/tau_pre : 1 (event-driven)
                                        dApost/dt=-Apost/tau_post : 1 (event-driven)
                                        du_f/dt = (U-u_f)/tau_f : 1 (event-driven)
                                        dx_d/dt = (1-x_d)/tau_d : 1 (event-driven)
                                        s_nmda_in2_post = int(not removed) * tmp_nmda_pre : 1 (summed)
                                        s_ampa_in2_post = int(not removed) * tmp_ampa_pre : 1 (summed)
                                        ''',
                      delay=condDelay,
                      on_pre='''
                                u_f += int(not removed) * U * (1 - u_f)
                                r_S = int(not removed) * u_f * x_d
                                x_d -= int(not removed) * r_S
                                s_ampa_post += int(not removed) * w_STDP
                                x_post += int(not removed) * w_STDP
                                tmp_ampa = int(not removed) * clip(s_ampa*r_S*w_ee, 0, 100*r_S*w_ee)
                                tmp_nmda = int(not removed) * clip(s_nmda*r_S*w_ee, 0, 100*r_S*w_ee)
                                Apre += dApre
                                w_STDP = int(not removed) * clip(w_STDP + Apost, 0, wmax)
                                Ca += int(not removed) * a_Ca
                                ''',
                     on_post='''
                                 Apost += dApost
                                 w_STDP = int(not removed) * clip(w_STDP + Apre, 0, wmax)
                            ''')
    
    con_ee.connect(condition='i!=j', p=0.1*multipliers[1,n])
    con_ee.u_f = U
    con_ee.x_d = 1
    con_ee.w_STDP = w_ee
    
    # Excitatory - Inhibitory connections (STP only)
    con_ei = Synapses(Pe, Pi, model = '''
                                        du_f/dt = (U-u_f)/tau_f : 1 (event-driven)
                                        dx_d/dt = (1-x_d)/tau_d : 1 (event-driven)
                                        s_nmda_in3_post = int(not removed) * tmp_nmda_pre : 1 (summed)
                                        s_ampa_in3_post = int(not removed) * tmp_ampa_pre : 1 (summed)
                                        ''',
                      delay=condDelay,
                      on_pre='''
                                u_f += int(not removed) * U * (1 - u_f)
                                r_S = int(not removed) * u_f * x_d
                                x_d -= int(not removed) * r_S
                                s_ampa_post += int(not removed) * w
                                x_post += int(not removed) * w
                                tmp_ampa = int(not removed) * clip(s_ampa*r_S*w_ei, 0, 100*r_S*w_ei)
                                tmp_nmda = int(not removed) * clip(s_nmda*r_S*w_ei, 0, 100*r_S*w_ei)
                                Ca += int(not removed) * a_Ca
                       ''')
    
    con_ei.connect(condition='i!=j', p=0.05)                   
    con_ei.u_f = U
    con_ei.x_d = 1
    con_ei.w = 1
    
    # Inhibitory - Excitatory connections (STP only)
    con_ie = Synapses(Pi, Pe, model = '''
                                        du_f/dt = (U-u_f)/tau_f : 1 (event-driven)
                                        dx_d/dt = (1-x_d)/tau_d : 1 (event-driven)
                                        ''', 
                      delay=condDelay,
                      on_pre='''
                                u_f += int(not removed) * U * (1 - u_f)
                                r_S = int(not removed) * u_f * x_d
                                x_d -= int(not removed) * r_S
                                s_gaba_post += int(not removed) * r_S * w
                                Ca += int(not removed) * a_Ca
                       ''')
    
    con_ie.connect(condition='i!=j', p=0.2)
    con_ie.u_f = U
    con_ie.x_d = 1
    con_ie.w = 1
    
    # Inhibitory - Inhibitory connections (STP only)
    con_ii = Synapses(Pi, Pi, model = '''
                                        du_f/dt = (U-u_f)/tau_f : 1 (event-driven)
                                        dx_d/dt = (1-x_d)/tau_d : 1 (event-driven)
                                        ''', 
                      delay=condDelay,
                      on_pre='''
                                u_f += int(not removed) * U * (1 - u_f)
                                r_S = int(not removed) * u_f * x_d
                                x_d -= int(not removed) * r_S
                                s_gaba_post += int(not removed) * r_S * w
                                Ca += int(not removed) * a_Ca
                       ''')
    
    con_ii.connect(condition='i!=j', p=0.2)                  
    con_ii.u_f = U
    con_ii.x_d = 1
    con_ii.w = 1
    
    
    #%% Initial Values
    
    # Excitatory initial values
    Pe.v = 'El_e + rand()*(Vth_e-El_e)'  # start each neuron from a different random voltage to show that whatever behavior occurs doens't depend on initial conditions
    Pe.s_ampa = 0
    Pe.s_gaba = 0
    Pe.s_nmda = 0
    Pe.Ca = 0
    Pe.I = I_on * np.random.normal(mu, sigma, 1).item()
    Pe.inputCurrent = multipliers[2,n]
    
    # Inhibitory initial values
    Pi.v = 'El_i + rand()*(Vth_i-El_i)' 
    Pi.s_gaba = 0
    Pi.Ca = 0
    Pi.s_ampa = 0
    Pi.s_nmda = 0
    Pi.I = I_on * np.random.normal(mu, sigma, 1).item()
    
    
    #%% Set up monitors
    
    # Excitatory neurons:
    exc_statemon = StateMonitor(Pe,['s_gaba', 's_nmda_in', 's_ampa_in'], record=True, dt=10*msecond)
    exc_statemon_selectV = StateMonitor(Pe,'v',record=list(np.random.randint(0,NE,10))) # record a random 10 neurons to check their behavior
    exc_spikemon = SpikeMonitor(Pe)
    
    # Inhibitory neurons:
    inh_statemon = StateMonitor(Pi,['s_gaba', 's_nmda_in', 's_ampa_in'], record=True, dt=10*msecond)
    inh_statemon_selectV = StateMonitor(Pi,'v',record=list(np.random.randint(0,NI,10))) # record a random 10 neurons to check their behavior
    inh_spikemon = SpikeMonitor(Pi)
    
    #%% Set up min and max bounds for the synaptic weight variables
    
    scale = ''
    
    for receptor in ['nmda','ampa']:
        for pop in ['Pe', 'Pi']:
            scale += pop+'.s_'+receptor+'_ = clip('+pop+'.s_'+receptor+', 0, 100)'
            scale += '\n'
    
    @network_operation()
    def s():
        exec(scale) # like an eval() statement in Matlab
    
    #%% Run stabilization
    print("Running stabilization phase...")    
    run(pre_simtime, report='text')
    
    #%% Run simulation
    print("Running simulation phase...")
    run(simtime, report='text')
    
    #%% Save data:    
    g_all = {
        "g_nmda_e" : float(g_nmda_e)*1e9, # all conductances in nS
        "g_nmda_i" : float(g_nmda_i)*1e9,
        "g_ampa_e" : float(g_ampa_e)*1e9, 
        "g_ampa_i" : float(g_ampa_i)*1e9,
        "g_gaba_e" : float(g_gaba_e)*1e9, 
        "g_gaba_i" : float(g_gaba_i)*1e9}
    
    E_all = {
        "E_nmda" : float(E_nmda)*1e3, # all potentials in mV
        "E_ampa" : float(E_ampa)*1e3,
        "E_gaba" : float(E_gaba)*1e3}
    
    variables_to_save = {
        "TimeStepInSeconds" : float(defaultclock.dt),
        "SimTimeInSeconds" : float(simtime),
        "StabilizationTimeInSeconds" : float(pre_simtime),
        "N" : N,
        "NE" : NE,
        "NI" : NI,
        "exc_v" : np.array(exc_statemon_selectV.v[-10000::,:])*1e3, # all potentials in mV
        "inh_v" : np.array(exc_statemon_selectV.v[-10000::,:])*1e3,
        "v_time" : np.array(exc_statemon_selectV.t[-10000::]),
        "exc_s_nmda_in": exc_statemon.s_nmda_in[-1000::,:], # for samples recorded every 10ms, take the last 10s
        "exc_s_ampa_in": exc_statemon.s_ampa_in[-1000::,:],
        "exc_s_gaba" : exc_statemon.s_gaba[-1000::,:],
        "inh_s_nmda_in": inh_statemon.s_nmda_in[-1000::,:],
        "inh_s_ampa_in": inh_statemon.s_ampa_in[-1000::,:],
        "inh_s_gaba" : inh_statemon.s_gaba[-1000::,:],
        "exc_spiketimes" : np.fromiter(map(float,exc_spikemon.t),float), # turn these from Brian objects (which suck) to floats (which don't)
        "inh_spiketimes" : np.fromiter(map(float,inh_spikemon.t),float),
        "exc_spikeindexes" : exc_spikemon.i[:],
        "inh_spikeindexes" : inh_spikemon.i[:],
        "g_all" : str(g_all),  # scipy.savemat won't save a dictionary, so convert this to a string to convert back once it's loaded
        "E_all" : str(E_all),
        # "inputNeurons" : inputNeurons[:],
        "readme" : "Once loaded, convet E_all and g_all back to dict with ast.literal_eval"}
    
    savename = str.format("Cypin OE - AMPA x{}, Connection x{}, Presynaptic Current x{} {}.mat",multipliers[0,n],multipliers[1,n],multipliers[2,n],floor(n/nConditions)+1)
    sio.savemat(savename,variables_to_save)
    
    # # Need to free up memory for next sim:
    # del variables_to_save, exc_statemon, inh_statemon, exc_spikemon, inh_spikemon
    # del inh_statemon_selectV, exc_statemon_selectV
    # del Pe, Pi, con_ee, con_ii, con_ei, con_ie, self_exc, self_inh
    # gc.collect()
    # print(f"Simulation complete in {round(SimTimer.stop()/60,2)} minutes.")






