
% This function loads control and cypin overexpressed simulations from 
% Python (cypinOverexpressionIO.py) that have input neurons that have been 
% stimulated and generates a functional connectivity matrix
% that is used to calculate global efficiency, community statistic (Q) - 
% modularity, and number of communities for the pre-training and 
% post-training segments. The values reported in the table are the 
% normalized to the pre-training segment. These analyses were used to 
% produce Figure 13 in Rodriguez, et al., Cytosolic PSD-95 Interactor 
% Regulates AMPA Receptor Signaling Independent of PSD-95 Binding. Network 
% Neurosci, 2020.
% 
% This code was developed by Erin D. Anderson and can be accessed at 
% https://www.seas.upenn.edu/~molneuro/

% Inputs: 
%   folderName: string referring to folder name containing the cypin overexpression *.mat files
%   overwrite: logical - whether or not to save and overwrite new spike analysis

% Outputs:
%   networkAnalysisNormIO: table containing the network analysis grouped by
%       condition and normalized to control


function networkAnalysisNormIO = cypinOverexpressionIONetworkAnalysis(folderName,overwrite)

%% Load files and generate FC matrices, do network analysis

fileNames = dir(folderName);
fileNames(1:2) = [];
for i = length(fileNames):-1:1
    if fileNames(i).isdir
        fileNames(i) = [];
    elseif ~strcmp(fileNames(i).name(end-3:end),'.mat')
        fileNames(i) = [];
    elseif ~contains(fileNames(i).name,"Cypin OE")
        fileNames(i) = [];
    elseif ~contains(fileNames(i).name,"Stimulus") % make sure getting stimulated data
        fileNames(i) = [];
    end
end

nSimulations = length(fileNames);

maxLagInSeconds = 20e-3;
binSizeInSeconds = 1e-3;

FC_pre = cell(nSimulations,1);
FC_post = cell(nSimulations,1);

GE_pre = zeros(nSimulations,1);
GE_post = zeros(nSimulations,1);
nCommunities_pre = zeros(nSimulations,1);
nCommunities_post = zeros(nSimulations,1);
Q_pre = zeros(nSimulations,1);
Q_post = zeros(nSimulations,1);

for ii = 1:nSimulations
    mat = matfile(fullfile(folderName,fileNames(ii).name));
    % Run analysis:
    pre_startStop = [0 mat.SimTimeInSeconds-mat.TimeStepInSeconds]; % start/stop for 1st (pre-training) epoch
    FC_pre{ii} = functionalConnectivityMatrix(mat.exc_spikeindexes,mat.exc_spiketimes,...
        mat.SimTimeInSeconds,mat.StabilizationTimeInSeconds,...
        mat.NE,maxLagInSeconds,binSizeInSeconds,pre_startStop);
    
    post_startStop = [mat.SimTimeInSeconds*2,...
        mat.SimTimeInSeconds*3-mat.TimeStepInSeconds]; % start/stop for 3rd (post-training) epoch
    FC_post{ii} = functionalConnectivityMatrix(mat.exc_spikeindexes,mat.exc_spiketimes,...
        mat.SimTimeInSeconds,mat.StabilizationTimeInSeconds,...
        mat.NE,maxLagInSeconds,binSizeInSeconds,post_startStop);
    
    [GE_pre(ii),nCommunities_pre(ii),Q_pre(ii)] = cypinNetworkAnalysis(FC_pre{ii});
    [GE_post(ii),nCommunities_post(ii),Q_post(ii)] = cypinNetworkAnalysis(FC_post{ii});
    
    if ~ismember('networkDataIO',who('-file',fullfile(folderName,fileNames(ii).name))) || overwrite
        % Save analysis results to the original mat file:
        networkDataIO = struct('FC',NaN);
        networkDataIO.FC_pre = FC_pre{ii};
        networkDataIO.GE_pre = GE_pre(ii);
        networkDataIO.nCommunities_pre = nCommunities_pre(ii);
        networkDataIO.Q_pre = Q_pre(ii);
        
        networkDataIO.FC_post = FC_post{ii};
        networkDataIO.GE_post = GE_post(ii);
        networkDataIO.nCommunities_post = nCommunities_post(ii);
        networkDataIO.Q_post = Q_post(ii);
        save(fullfile(folderName,fileNames(ii).name),'networkDataIO','-append');
        clearvars networkDataIO
    end
end

%% Sort data into matrices grouped by condition:

% Get conditions:

AMPA_Multiplier = zeros(nSimulations,1);
Connection_Multiplier = zeros(nSimulations,1);
Ipre_Multiplier = zeros(nSimulations,1);

for i = 1:nSimulations
    [StartIndexAMPA,EndIndexAMPA] = regexp(fileNames(i).name,'A x.{1,3},');
    [StartIndexCD,EndIndexCD] = regexp(fileNames(i).name,'n x.{1,4},');
    [StartIndexIpre,EndIndexIpre] = regexp(fileNames(i).name,'t x.{1,4},');
    AMPA_Multiplier(i) = str2double(fileNames(i).name(StartIndexAMPA+3:EndIndexAMPA-1));
    Connection_Multiplier(i) = str2double(fileNames(i).name(StartIndexCD+3:EndIndexCD-1));
    Ipre_Multiplier(i) = str2double(fileNames(i).name(StartIndexIpre+3:EndIndexIpre-1));
end

[combos,~,ind] = unique([AMPA_Multiplier Connection_Multiplier,...
    Ipre_Multiplier],'rows');
RowsForEachCombo = accumarray(ind, find(ind), [], @(rows){rows});
TempLengths = [];
for i = 1:length(RowsForEachCombo)
    TempLengths = [TempLengths length(RowsForEachCombo{i})];
end
biggestLength = max(TempLengths);
nCombos = length(combos);

% Group conditions:

FC_pre_grouped = cell(biggestLength,nCombos);
FC_post_grouped = cell(biggestLength,nCombos);

GE_pre_grouped = NaN(biggestLength,nCombos);
GE_post_grouped = NaN(biggestLength,nCombos);
nCommunities_pre_grouped = NaN(biggestLength,nCombos);
nCommunities_post_grouped = NaN(biggestLength,nCombos);
Q_pre_grouped = NaN(biggestLength,nCombos);
Q_post_grouped = NaN(biggestLength,nCombos);

for ii = 1:length(RowsForEachCombo) 
    FC_pre_grouped(1:length(RowsForEachCombo{ii}),ii) = FC_pre(RowsForEachCombo{ii});
    FC_post_grouped(1:length(RowsForEachCombo{ii}),ii) = FC_post(RowsForEachCombo{ii});

    GE_pre_grouped(1:length(RowsForEachCombo{ii}),ii) = GE_pre(RowsForEachCombo{ii});
    GE_post_grouped(1:length(RowsForEachCombo{ii}),ii) = GE_post(RowsForEachCombo{ii});
    nCommunities_pre_grouped(1:length(RowsForEachCombo{ii}),ii) = nCommunities_pre(RowsForEachCombo{ii});
    nCommunities_post_grouped(1:length(RowsForEachCombo{ii}),ii) = nCommunities_post(RowsForEachCombo{ii});
    Q_pre_grouped(1:length(RowsForEachCombo{ii}),ii) = Q_pre(RowsForEachCombo{ii});
    Q_post_grouped(1:length(RowsForEachCombo{ii}),ii) = Q_post(RowsForEachCombo{ii});
end

%% Normalize:

% % Pull out control:
% [~,controlCombo] = ismember(combos,[1,1,1],'rows'); % find column that corresponds to unmodified control
% if sum(controlCombo) == 1 % if you have a control case, pull out the controls for normalization
%     controlCombo = logical(controlCombo); % to make indexing easier
%     
%     GE_pre_control = GE_pre_grouped(:,controlCombo);
%     nCommunities_pre_control = nCommunities_pre_grouped(:,controlCombo);
%     Q_pre_control = Q_pre_grouped(:,controlCombo);
%     
%     GE_post_control = GE_post_grouped(:,controlCombo);
%     nCommunities_post_control = nCommunities_post_grouped(:,controlCombo);
%     Q_post_control = Q_post_grouped(:,controlCombo);
% else % leave as 1 so no normalization happens
%     GE_pre_control = 1;
%     nCommunities_pre_control = 1;
%     Q_pre_control = 1;
%     
%     GE_post_control = 1;
%     nCommunities_post_control = 1;
%     Q_post_control = 1;
% end

% Normalize:
GE_pre_norm = GE_pre_grouped./nanmean(GE_pre_grouped);
nCommunities_pre_norm = nCommunities_pre_grouped./nanmean(nCommunities_pre_grouped);
Q_pre_norm = Q_pre_grouped./nanmean(Q_pre_grouped);

GE_post_norm = GE_post_grouped./nanmean(GE_pre_grouped);
nCommunities_post_norm = nCommunities_post_grouped./nanmean(nCommunities_pre_grouped);
Q_post_norm = Q_post_grouped./nanmean(Q_pre_grouped);

%% Make summary table:

columnNames = {'Name','AMPA_Multiplier','Connection_Multiplier','Ipre_Multiplier',...
    'FC_pre','FC_post','GE_pre','GE_post','nCommunities_pre','nCommunities_post',...
    'Q_pre','Q_post'};
columnTypes = {'string','double','double','double','double','double',...
    'double','double','double','double','double','double'};
networkAnalysisNormIO = table('Size',[nCombos length(columnNames)],...
    'VariableNames',columnNames,'VariableTypes',columnTypes);

comboNames = cell(nCombos,1);
for ii = 1:nCombos
    comboNames{ii} = ['AMPA x' num2str(combos(ii,1)) ', Connection x' ...
        num2str(combos(ii,2)) ', I_pre x' num2str(combos(ii,3))];
end

networkAnalysisNormIO.Name = comboNames;
networkAnalysisNormIO.AMPA_Multiplier = combos(:,1);
networkAnalysisNormIO.Connection_Multiplier = combos(:,2);
networkAnalysisNormIO.Ipre_Multiplier = combos(:,3);
networkAnalysisNormIO.FC_pre = FC_pre_grouped'; % not normalized
networkAnalysisNormIO.FC_post = FC_post_grouped'; % not normalized
networkAnalysisNormIO.GE_pre = GE_pre_norm';
networkAnalysisNormIO.GE_post = GE_post_norm';
networkAnalysisNormIO.nCommunities_pre = nCommunities_pre_norm';
networkAnalysisNormIO.nCommunities_post = nCommunities_post_norm';
networkAnalysisNormIO.Q_pre = Q_pre_norm';
networkAnalysisNormIO.Q_post = Q_post_norm';


