
% This function loads control and cypin overexpressed simulations from 
% Python (cypinOverexpressionIO.py) that have input neurons that have been 
% stimulated and calculates spike rate for input and output 
% neurons post-training, normalized to the pre-training control segment. 
% These analyses were used to produce Figure 12 in Rodriguez, et al., 
% Cytosolic PSD-95 Interactor Regulates AMPA Receptor Signaling Independent 
% of PSD-95 Binding. Network Neurosci, 2020.
% 
% This code was developed by Erin D. Anderson and Nicholas F. Vigilante and 
% can be accessed at https://www.seas.upenn.edu/~molneuro/

% Inputs: 
%   folderName: string referring to folder name containing the cypin overexpression *.mat files
%   overwrite: logical - whether or not to save and overwrite new spike analysis

% Outputs:
%   spikeAnalysisNormIO: table containing the spike analysis for IO experiment
%       grouped by condition and normalized to control

function spikeAnalysisNormIO = cypinOverexpressionIOSpikeAnalysis(folderName,overwrite)

% Collect files to run:
fileNames = dir(folderName);
fileNames(1:2) = [];

for i = length(fileNames):-1:1
    if fileNames(i).isdir
        fileNames(i) = [];
    elseif ~strcmp(fileNames(i).name(end-3:end),'.mat')
        fileNames(i) = [];
    elseif ~contains(fileNames(i).name,"Cypin OE")
        fileNames(i) = [];
    elseif ~contains(fileNames(i).name,"Stimulus") % make sure getting stimulated data
        fileNames(i) = [];
    end
end

firingRateControl = zeros(length(fileNames),1);
firingRateTestingInput = zeros(length(fileNames),1);
firingRateTestingOutput  = zeros(length(fileNames),1);
firingRateControlInput = zeros(length(fileNames),1);
firingRateControlOutput = zeros(length(fileNames),1);

for ii = 1:length(fileNames)
    mat = matfile(fullfile(folderName,fileNames(ii).name));
    % Run analysis:
    [firingRateControl(ii),firingRateControlInput(ii),firingRateControlOutput(ii),...
        firingRateTestingInput(ii),firingRateTestingOutput(ii)] = ...
        firingRatesIO(mat.exc_spiketimes,mat.exc_spikeindexes,...
        mat.SimTimeInSeconds,mat.StabilizationTimeInSeconds,mat.inputNeurons);
    % If IOData struct doesn't already exist for this file, or you want to overwrite it, save a new one
    if ~ismember('spikeDataIO',who('-file',fullfile(folderName,fileNames(ii).name))) || overwrite
        % Save analysis results to the original mat file:
        spikeDataIO = struct('firingRateControl',NaN);
        spikeDataIO.firingRateControl = firingRateControl(ii);
        spikeDataIO.firingRateControlInput = firingRateControlInput(ii);
        spikeDataIO.firingRateControlOutput = firingRateControlOutput(ii);
        spikeDataIO.firingRateTestingInput = firingRateTestingInput(ii);
        spikeDataIO.firingRateTestingOutput = firingRateTestingOutput(ii);
        save(fullfile(folderName,fileNames(ii).name),'spikeDataIO','-append');
        clearvars spikeDataIO
    end
end

%% Make table:
spikeTableIO = table('Size',[length(fileNames) 12],'VariableNames',{'FileName',...
    'AMPA_Multiplier','Connection_Multiplier','Ipre_Multiplier',...
    'firingRateControl','firingRateTestingInput','firingRateTestingOutput',...
    'firingRateControlNorm','firingRateTestingInputNorm',...
    'firingRateTestingOutputNorm','firingRateControlInput',...
    'firingRateControlOutput'},...
    'VariableTypes',{'string','double','double','double','double','double',...
    'double','double','double','double','double','double'});
spikeTableIO.firingRateControl = firingRateControl;
spikeTableIO.firingRateTestingInput = firingRateTestingInput;
spikeTableIO.firingRateTestingOutput = firingRateTestingOutput;
spikeTableIO.firingRateControlNorm = firingRateControl./firingRateControl;
spikeTableIO.firingRateTestingInputNorm = firingRateTestingInput./firingRateControl;
spikeTableIO.firingRateTestingOutputNorm = firingRateTestingOutput./firingRateControl;
spikeTableIO.firingRateControlInput = firingRateControlInput;
spikeTableIO.firingRateControlOutput = firingRateControlOutput;

for i = 1:length(fileNames)
    spikeTableIO.FileName{i} = fileNames(i).name;
    [StartIndexAMPA,EndIndexAMPA] = regexp(fileNames(i).name,'A x.{1,3},');
    [StartIndexCD,EndIndexCD] = regexp(fileNames(i).name,'n x.{1,4},');
    [StartIndexIpre,EndIndexIpre] = regexp(fileNames(i).name,'t x.{1,3},');
    [StartIndexS,EndIndexS] = regexp(fileNames(i).name,'s x.{1,3}');
    spikeTableIO.AMPA_Multiplier(i) = str2double(fileNames(i).name(StartIndexAMPA+3:EndIndexAMPA-1));
    spikeTableIO.Connection_Multiplier(i) = str2double(fileNames(i).name(StartIndexCD+3:EndIndexCD-1));
    spikeTableIO.Ipre_Multiplier(i) = str2double(fileNames(i).name(StartIndexIpre+3:EndIndexIpre-1));
    spikeTableIO.Stimulus_Multiplier(i) = str2double(fileNames(i).name(StartIndexS+3:EndIndexS-1));
end

%% Group by condition:

[combos,~,ind] = unique([spikeTableIO.AMPA_Multiplier spikeTableIO.Connection_Multiplier,...
    spikeTableIO.Ipre_Multiplier spikeTableIO.Stimulus_Multiplier],'rows');
RowsForEachCombo = accumarray(ind, find(ind), [], @(rows){rows});
TempLengths = [];
for i = 1:length(RowsForEachCombo)
    TempLengths = [TempLengths length(RowsForEachCombo{i})];
end
biggestLength = max(TempLengths);
nCombos = size(combos,1);

FRControl_grouped = NaN(biggestLength,nCombos);
FRControlI_grouped = NaN(biggestLength,nCombos);
FRControlO_grouped = NaN(biggestLength,nCombos);
FRTestingI_grouped = NaN(biggestLength,nCombos);
FRTestingO_grouped = NaN(biggestLength,nCombos);
for ii = 1:length(RowsForEachCombo)
    FRControl_grouped(1:length(RowsForEachCombo{ii}),ii) = firingRateControl(RowsForEachCombo{ii});
    FRControlI_grouped(1:length(RowsForEachCombo{ii}),ii) = firingRateControlInput(RowsForEachCombo{ii});
    FRControlO_grouped(1:length(RowsForEachCombo{ii}),ii) = firingRateControlOutput(RowsForEachCombo{ii});
    FRTestingI_grouped(1:length(RowsForEachCombo{ii}),ii) = firingRateTestingInput(RowsForEachCombo{ii});
    FRTestingO_grouped(1:length(RowsForEachCombo{ii}),ii) = firingRateTestingOutput(RowsForEachCombo{ii});
end

%% Normalize to control epoch

firingRateControlNorm = FRControl_grouped./mean(FRControl_grouped,1);
firingRateTestingInputNorm = FRTestingI_grouped./mean(FRControl_grouped,1);
firingRateTestingOutputNorm = FRTestingO_grouped./mean(FRControl_grouped,1);

%% Make summary table:

columnNames = {'Name','AMPA_Multiplier','Connection_Multiplier','Ipre_Multiplier',...
    'firingRateControl','firingRateTestingInput',...
    'firingRateTestingOutput','signalFidelity'};
columnTypes = {'string','double','double','double','double','double',...
    'double','double'};
spikeAnalysisNormIO = table('Size',[nCombos length(columnNames)],...
    'VariableNames',columnNames,'VariableTypes',columnTypes);

comboNames = cell(nCombos,1);
for ii = 1:nCombos
    comboNames{ii} = ['AMPA x' num2str(combos(ii,1)) ', Connection x' ...
        num2str(combos(ii,2)) ', I_pre x' num2str(combos(ii,3))];
end

spikeAnalysisNormIO.Name = comboNames;
spikeAnalysisNormIO.AMPA_Multiplier = combos(:,1);
spikeAnalysisNormIO.Connection_Multiplier = combos(:,2);
spikeAnalysisNormIO.Ipre_Multiplier = combos(:,3);
spikeAnalysisNormIO.firingRateControl = firingRateControlNorm';
spikeAnalysisNormIO.firingRateTestingInput = firingRateTestingInputNorm';
spikeAnalysisNormIO.firingRateTestingOutput = firingRateTestingOutputNorm';
spikeAnalysisNormIO.signalFidelity = mean(firingRateTestingOutputNorm,1)./...
    mean(firingRateTestingInputNorm,1)'; % output/input
