
% This function loads control and cypin overexpressed networks from Python 
% (cypinOverexpression.py) and generates a functional connectivity matrix
% that is used to calculate global efficiency, community statistic (Q) - 
% modularity, and number of communities. These analyses were used to 
% produce Figure 11 in Rodriguez, et al., Cytosolic PSD-95 Interactor 
% Regulates AMPA Receptor Signaling Independent of PSD-95 Binding. Network 
% Neurosci, 2020.
% 
% This code was developed by Erin D. Anderson and can be accessed at 
% https://www.seas.upenn.edu/~molneuro/

% Inputs: 
%   folderName: string referring to folder name containing the cypin overexpression *.mat files
%   overwrite: logical - whether or not to save and overwrite new spike analysis

% Outputs:
%   networkAnalysisNorm: table containing the network analysis grouped by
%       condition and normalized to control

function networkAnalysisNorm = cypinOverexpressionNetworkAnalysis(folderName,overwrite)

%% Load data and generate FC matrices, analyze network:
fileNames = dir(folderName);
fileNames(1:2) = [];
for i = length(fileNames):-1:1
    if fileNames(i).isdir
        fileNames(i) = [];
    elseif ~strcmp(fileNames(i).name(end-3:end),'.mat')
        fileNames(i) = [];
    elseif ~contains(fileNames(i).name,"Cypin OE")
        fileNames(i) = [];
    elseif contains(fileNames(i).name,"Stimulus") % make sure you're not getting any stimulated data
        fileNames(i) = [];
    end
end

nSimulations = length(fileNames);

FC = cell(nSimulations,1);
GE = zeros(nSimulations,1);
nCommunities = zeros(nSimulations,1);
Q = zeros(nSimulations,1);

maxLagInSeconds = 20e-3;
binSizeInSeconds = 1e-3;

for ii = 1:nSimulations
    mat = matfile(fullfile(folderName,fileNames(ii).name));
    % Run analysis:
    FC{ii} = functionalConnectivityMatrix(mat.exc_spikeindexes,mat.exc_spiketimes,...
        mat.SimTimeInSeconds,mat.StabilizationTimeInSeconds,...
        mat.NE,maxLagInSeconds,binSizeInSeconds);
    [GE(ii),nCommunities(ii),Q(ii)] = networkAnalysis(FC{ii});
    
    % If networkData doesn't already exist for this file, or you want to 
    % overwrite it, save a new one
    if ~ismember('networkData',who('-file',fullfile(folderName,fileNames(ii).name))) || overwrite
        % Save analysis results to the original mat file:
        networkData = struct('FC',NaN);
        networkData.FC = FC{ii};
        networkData.GE = GE(ii);
        networkData.nCommunities = nCommunities(ii);
        networkData.Q = Q(ii);
        save(fullfile(folderName,fileNames(ii).name),'networkData','-append');
        clearvars networkData
    end
end

%% Sort data into matrices grouped by condition:

% Get conditions:

AMPA_Multiplier = zeros(nSimulations,1);
Connection_Multiplier = zeros(nSimulations,1);
Ipre_Multiplier = zeros(nSimulations,1);

for i = 1:nSimulations
    [StartIndexAMPA,EndIndexAMPA] = regexp(fileNames(i).name,'A x.{1,3},');
    [StartIndexCD,EndIndexCD] = regexp(fileNames(i).name,'n x.{1,4},');
    [StartIndexIpre,EndIndexIpre] = regexp(fileNames(i).name,'t x.{1,4}');
    AMPA_Multiplier(i) = str2double(fileNames(i).name(StartIndexAMPA+3:EndIndexAMPA-1));
    Connection_Multiplier(i) = str2double(fileNames(i).name(StartIndexCD+3:EndIndexCD-1));
    Ipre_Multiplier(i) = str2double(fileNames(i).name(StartIndexIpre+3:EndIndexIpre-1));
end

[combos,~,ind] = unique([AMPA_Multiplier Connection_Multiplier,...
    Ipre_Multiplier],'rows');
RowsForEachCombo = accumarray(ind, find(ind), [], @(rows){rows});
TempLengths = [];
for i = 1:length(RowsForEachCombo)
    TempLengths = [TempLengths length(RowsForEachCombo{i})];
end
biggestLength = max(TempLengths);
nCombos = length(combos);

FC_grouped = cell(biggestLength,nCombos);
GE_grouped = NaN(biggestLength,nCombos);
nCommunities_grouped = NaN(biggestLength,nCombos);
Q_grouped = NaN(biggestLength,nCombos);
for ii = 1:length(RowsForEachCombo)
    FC_grouped(1:length(RowsForEachCombo{ii}),ii) = FC(RowsForEachCombo{ii});
    GE_grouped(1:length(RowsForEachCombo{ii}),ii) = GE(RowsForEachCombo{ii});
    nCommunities_grouped(1:length(RowsForEachCombo{ii}),ii) = nCommunities(RowsForEachCombo{ii});
    Q_grouped(1:length(RowsForEachCombo{ii}),ii) = Q(RowsForEachCombo{ii});
end

%% Normalize to control network data:

[~,controlCombo] = ismember(combos,[1,1,1],'rows'); % find column that corresponds to unmodified control
if sum(controlCombo) == 1 % if you have a control case, pull out the controls for normalization
    controlCombo = logical(controlCombo); % to make indexing easier
    
    GEcontrol = GE_grouped(:,controlCombo);
    nCommunitiesControl = nCommunities_grouped(:,controlCombo);
    Qcontrol = Q_grouped(:,controlCombo);
else % leave as 1 so no normalization happens
    GEcontrol = 1;
    nCommunitiesControl = 1;
    Qcontrol = 1;
end

% Normalize:
GEnorm = GE_grouped/nanmean(GEcontrol);
nCommunitiesNorm = nCommunities_grouped/nanmean(nCommunitiesControl);
Qnorm = Q_grouped/nanmean(Qcontrol);

%% Make summary table:

columnNames = {'Name','AMPA_Multiplier','Connection_Multiplier','Ipre_Multiplier',...
    'FC','GE','nCommunities','Q'};
columnTypes = {'string','double','double','double','double','double',...
    'double','double'};
networkAnalysisNorm = table('Size',[nCombos length(columnNames)],...
    'VariableNames',columnNames,'VariableTypes',columnTypes);

comboNames = cell(nCombos,1);
for ii = 1:nCombos
    comboNames{ii} = ['AMPA x' num2str(combos(ii,1)) ', Connection x' ...
        num2str(combos(ii,2)) ', I_pre x' num2str(combos(ii,3))];
end

networkAnalysisNorm.Name = comboNames;
networkAnalysisNorm.AMPA_Multiplier = combos(:,1);
networkAnalysisNorm.Connection_Multiplier = combos(:,2);
networkAnalysisNorm.Ipre_Multiplier = combos(:,3);
networkAnalysisNorm.FC = FC_grouped'; % not normalized
networkAnalysisNorm.GE = GEnorm';
networkAnalysisNorm.nCommunities = nCommunitiesNorm';
networkAnalysisNorm.Q = Qnorm';
