
% This function loads control and cypin overexpressed networks from Python 
% (cypinOverexpression.py) and calculates spike rate, inter-spike interval
% (ISI), Fano factor (FF), coefficient of variation (CV), and network spike
% (burst) rate (NS rate). These analyses were used to produce Figure 10 
% in Rodriguez, et al., Cytosolic PSD-95 Interactor Regulates AMPA 
% Receptor Signaling Independent of PSD-95 Binding. Network Neurosci, 2020.
% 
% This code was developed by Erin D. Anderson and Nicholas F. Vigilante and 
% can be accessed at https://www.seas.upenn.edu/~molneuro/

% Inputs: 
%   folderName: string referring to folder name containing the cypin overexpression *.mat files
%   overwrite: logical - whether or not to save and overwrite new spike analysis

% Outputs:
%   spikeAnalysisNorm: table containing the spike analysis grouped by
%       condition and normalized to control

function spikeAnalysisNorm = cypinOverexpressionSpikeAnalysis(folderName,overwrite)

%% Load and Analyze Data:
fileNames = dir(folderName);
fileNames(1:2) = [];
for i = length(fileNames):-1:1 % get file list and ensure it is only Cypin OE files
    if fileNames(i).isdir
        fileNames(i) = [];
    elseif ~strcmp(fileNames(i).name(end-3:end),'.mat')
        fileNames(i) = [];
    elseif ~contains(fileNames(i).name,"Cypin OE")
        fileNames(i) = [];
    elseif contains(fileNames(i).name,"Stimulus") % make sure you're not getting any stimulated data
        fileNames(i) = [];
    end
end

isi_avg = zeros(length(fileNames),1);
isi_std = zeros(length(fileNames),1);
insi_avg = zeros(length(fileNames),1);
insi_std = zeros(length(fileNames),1);
spikeRate_avg = zeros(length(fileNames),1);
spikeRate_std = zeros(length(fileNames),1);
NSrate = zeros(length(fileNames),1); % network spike rate AKA burst rate
CV = zeros(length(fileNames),1);
FF = zeros(length(fileNames),1);

for ii = 1:length(fileNames)
    mat = matfile(fullfile(folderName,fileNames(ii).name));
    % Run analysis:
    [isi_avg(ii),isi_std(ii),spikeRate_avg(ii),spikeRate_std(ii),CV(ii)] = ...
        ISI(mat.exc_spikeindexes,mat.exc_spiketimes,mat.SimTimeInSeconds,...
        mat.StabilizationTimeInSeconds);
    [insi_avg(ii),insi_std(ii),~,NSrate(ii),~] = ...
        INSI(mat.exc_spiketimes,mat.SimTimeInSeconds,mat.StabilizationTimeInSeconds);
    FF(ii) = fanoFactor(mat.exc_spiketimes,mat.SimTimeInSeconds,...
        mat.StabilizationTimeInSeconds);
    
    % If spikeData struct doesn't already exist for this file, or you want 
    % to overwrite it, save a new one
    if ~ismember('spikeData',who('-file',fullfile(folderName,fileNames(ii).name))) || overwrite
        % Save analysis results to the original mat file:
        spikeData = struct('isi_avg',NaN,'isi_std',NaN,'spikeCount',NaN,'FF',NaN);
        spikeData.isi_avg = isi_avg(ii);
        spikeData.isi_std = isi_std(ii);
        spikeData.insi_avg = insi_avg(ii);
        spikeData.insi_std = insi_std(ii);
        spikeData.spikeRate_avg = spikeRate_avg(ii);
        spikeData.spikeRate_std = spikeRate_std(ii);
        spikeData.NSrate = NSrate(ii); % network spike rate AKA burst rate
        spikeData.CV = CV(ii);
        spikeData.FF = FF(ii);
        save(fullfile(folderName,fileNames(ii).name),'spikeData','-append');
        clearvars spikeData
    end
end

%% Make table of raw network stats:
spikeTable = table('Size',[length(fileNames) 13],'VariableNames',{'FileName',...
    'AMPA_Multiplier','Connection_Multiplier','Ipre_Multiplier','insi_avg',...
    'insi_std','isi_avg','isi_std','spikeRate_avg','spikeRate_std','NSrate','CV',...
    'FF'},'VariableTypes',{'string','double','double','double','double','double',...
    'double','double','double','double','double','double','double'});
spikeTable.isi_avg = isi_avg;
spikeTable.isi_std = isi_std;
spikeTable.insi_avg = insi_avg;
spikeTable.insi_std = insi_std;
spikeTable.spikeRate_avg = spikeRate_avg;
spikeTable.spikeRate_std = spikeRate_std;
spikeTable.NSrate = NSrate;
spikeTable.CV = CV;
spikeTable.FF = FF;

for i = 1:length(fileNames)
    spikeTable.FileName{i} = fileNames(i).name;
    [StartIndexAMPA,EndIndexAMPA] = regexp(fileNames(i).name,'A x.{1,3},');
    [StartIndexCD,EndIndexCD] = regexp(fileNames(i).name,'n x.{1,4},');
    [StartIndexIpre,EndIndexIpre] = regexp(fileNames(i).name,'t x.{1,4}');
    spikeTable.AMPA_Multiplier(i) = str2double(fileNames(i).name(StartIndexAMPA+3:EndIndexAMPA-1));
    spikeTable.Connection_Multiplier(i) = str2double(fileNames(i).name(StartIndexCD+3:EndIndexCD-1));
    spikeTable.Ipre_Multiplier(i) = str2double(fileNames(i).name(StartIndexIpre+3:EndIndexIpre-1));
end

%% Sort data from table into matrices grouped by condition:
[combos,~,ind] = unique([spikeTable.AMPA_Multiplier spikeTable.Connection_Multiplier,...
    spikeTable.Ipre_Multiplier],'rows');
RowsForEachCombo = accumarray(ind, find(ind), [], @(rows){rows});
TempLengths = [];
for i = 1:length(RowsForEachCombo)
    TempLengths = [TempLengths length(RowsForEachCombo{i})];
end
biggestLength = max(TempLengths);
nCombos = length(combos);

ISIavg_grouped = NaN(biggestLength,nCombos);
ISIstd_grouped = NaN(biggestLength,nCombos);
INSIavg_grouped = NaN(biggestLength,nCombos);
INSIstd_grouped = NaN(biggestLength,nCombos);
SR_grouped = NaN(biggestLength,nCombos);
SRstd_grouped = NaN(biggestLength,nCombos);
NSrate_grouped = NaN(biggestLength,nCombos);
CV_grouped = NaN(biggestLength,nCombos);
FF_grouped = NaN(biggestLength,nCombos);
for ii = 1:length(RowsForEachCombo)
    ISIavg_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.isi_avg(RowsForEachCombo{ii});
    ISIstd_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.isi_std(RowsForEachCombo{ii});
    
    INSIavg_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.insi_avg(RowsForEachCombo{ii});
    INSIstd_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.insi_std(RowsForEachCombo{ii});
    
    SR_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.spikeRate_avg(RowsForEachCombo{ii});
    SRstd_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.spikeRate_std(RowsForEachCombo{ii});
    
    NSrate_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.NSrate(RowsForEachCombo{ii});
    
    CV_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.CV(RowsForEachCombo{ii});
    FF_grouped(1:length(RowsForEachCombo{ii}),ii) = spikeTable.FF(RowsForEachCombo{ii});
end

%% Normalize to control network data:

[~,controlCombo] = ismember(combos,[1,1,1],'rows'); % find column that corresponds to unmodified control
if sum(controlCombo) == 1 % if you have a control case, pull out the controls for normalization
    controlCombo = logical(controlCombo); % to make indexing easier
    
    ISIcontrol = ISIavg_grouped(:,controlCombo);
    INSIcontrol = INSIavg_grouped(:,controlCombo);
    SRcontrol = SR_grouped(:,controlCombo);
    NSRcontrol = NSrate_grouped(:,controlCombo);
    CVcontrol = CV_grouped(:,controlCombo);
    FFcontrol = FF_grouped(:,controlCombo);
else % leave as 1 so no normalization happens
    ISIcontrol = 1;
    INSIcontrol = 1;
    SRcontrol = 1;
    NSRcontrol = 1;
    CVcontrol = 1;
    FFcontrol = 1;
end

% Normalize:
ISIavg = ISIavg_grouped/nanmean(ISIcontrol);
ISIstd = ISIstd_grouped/nanmean(ISIcontrol);
INSIavg = INSIavg_grouped/nanmean(INSIcontrol);
INSIstd = INSIstd_grouped/nanmean(INSIcontrol);
SR = SR_grouped/nanmean(SRcontrol);
SRstd = SRstd_grouped/nanmean(SRcontrol);
NSR = NSrate_grouped/nanmean(NSRcontrol);  % network spike rate AKA burst rate
CV = CV_grouped/nanmean(CVcontrol);
FF = FF_grouped/nanmean(FFcontrol);

%% Make summary table:

columnNames = {'Name','AMPA_Multiplier','Connection_Multiplier','Ipre_Multiplier',...
    'ISIavg','ISIstd','INSIavg','INSIstd','SRavg','SRstd','NSR','CV','FF'};
columnTypes = {'string','double','double','double','double','double',...
    'double','double','double','double','double','double','double'};
spikeAnalysisNorm = table('Size',[nCombos length(columnNames)],...
    'VariableNames',columnNames,'VariableTypes',columnTypes);

comboNames = cell(nCombos,1);
for ii = 1:nCombos
    comboNames{ii} = ['AMPA x' num2str(combos(ii,1)) ', Connection x' ...
        num2str(combos(ii,2)) ', I_pre x' num2str(combos(ii,3))];
end

spikeAnalysisNorm.Name = comboNames;
spikeAnalysisNorm.AMPA_Multiplier = combos(:,1);
spikeAnalysisNorm.Connection_Multiplier = combos(:,2);
spikeAnalysisNorm.Ipre_Multiplier = combos(:,3);
spikeAnalysisNorm.ISIavg = ISIavg';
spikeAnalysisNorm.ISIstd = ISIstd';
spikeAnalysisNorm.INSIavg = INSIavg';
spikeAnalysisNorm.INSIstd = INSIstd';
spikeAnalysisNorm.SRavg = SR';
spikeAnalysisNorm.SRstd = SRstd';
spikeAnalysisNorm.NSR = NSR'; % network spike rate AKA burst rate
spikeAnalysisNorm.CV = CV';
spikeAnalysisNorm.FF = FF';

