
% Function by EA, adapted from Gabrieli, et al (unpublished) - NMDAR
% alterations after mTBI induce deficits in memory acquisition and recall

% This function takes in the exc_spiketimes and exc_spikeindexes, the 
% length of each segment of the simulation, and which neurons are "input" 
% neurons to identify the firing rate for control period and testing period
% for input, output and hidden layer neurons. It also reports which neurons 
% are the output neurons and which neurons are "hidden layer" neurons

% It assumes 3 SimTime length epochs - a control segment, a training
% segment, and a testing segment

% Outputs:
%   firingRateControl: the firing rate for ALL neurons in the first
%       (control) epoch
%   firingRateControlInput: the firing rate for INPUT neurons in the first
%       (control) epoch
%   firingRateControlOutput: the firing rate for OUTPUT neurons in the
%       first (contorl) epoch
%   firingRateTestingInput: the firing rate for the INPUT neurons in the
%       third (testing) epoch
%   firingRateTestingOutput: the firing rate for the OUTPUT neurons in the
%       third (testing) epoch
%   outputNeurons: indexes for the output neurons (any neuron that isn't
%       input)


function [firingRateControl,firingRateControlInput,firingRateControlOutput,...
    firingRateTestingInput,firingRateTestingOutput,outputNeurons] = ...
    firingRatesIO(spiketimes,spikeindexes,SimTime,StabilizationTime,inputNeurons)

%% Figure out which spikes correspond to which neurons in each time segment:
controlTempIndexes = find(spiketimes > StabilizationTime &...
    spiketimes < SimTime+StabilizationTime);
controlSpikes = spiketimes(controlTempIndexes);
controlIndexes = spikeindexes(controlTempIndexes);

trainingTempIndexes = find(spiketimes >= SimTime+StabilizationTime &...
    spiketimes < SimTime*2+StabilizationTime);
trainingSpikes = spiketimes(trainingTempIndexes);
trainingIndexes = spikeindexes(trainingTempIndexes);

testingTempIndexes = find(spiketimes >= SimTime*2+StabilizationTime &...
    spiketimes < SimTime*3+StabilizationTime);
testingSpikes = spiketimes(testingTempIndexes);
testingIndexes = spikeindexes(testingTempIndexes);

%% Calculate the nSpikes for each neuron in the section:
nNeurons = length(unique(spikeindexes));
controlSpikesNeuronal = zeros(nNeurons,1);
trainingSpikesNeuronal = zeros(nNeurons,1);
testingSpikesNeuronal = zeros(nNeurons,1);
for ii = 1:nNeurons
    controlSpikesNeuronal(ii) = sum(controlSpikes(controlIndexes == ii-1));
    trainingSpikesNeuronal(ii) = sum(trainingSpikes(trainingIndexes == ii-1));
    testingSpikesNeuronal(ii) = sum(testingSpikes(testingIndexes == ii-1));
end

%% ID the output neurons, hidden layer neurons
outputNeurons = 1:nNeurons;
outputNeurons(inputNeurons+1) = []; % get rid of the input neurons - they can't also be output neurons

%% gather firing rates to be put in bar plot

inputNeurons = inputNeurons'+1; % +1 to acct for Python --> Matlab

firingRateControl = mean(controlSpikesNeuronal)/SimTime;
firingRateControlInput = mean(controlSpikesNeuronal(inputNeurons))/SimTime;
firingRateControlOutput = mean(controlSpikesNeuronal(outputNeurons))/SimTime;
firingRateTestingInput = mean(testingSpikesNeuronal(inputNeurons))/SimTime;
firingRateTestingOutput = mean(testingSpikesNeuronal(outputNeurons))/SimTime;

end

