
% Function by EA, developed in conjunction with KO

% This function takes in the spiketimes and the length of the simulation to
% calculate the ISI, spike rate, and CV for every spike per neuron (electrode) 

% Briefly, this code takes in each neuron's spikes occuring within the
% simulation period and bins them before finding the cross-correlation
% between pairs of neurons with a prescribed lag time.

% Inputs:
%   spikeindexes: vector indicating which neuron is firing
%   spiketimes: vector indicating at what time that neuron fires
%   SimTimeInSeconds: length of the simulation period [s]
%   StabilizationTimeInSeconds: time before simulation period to omit [s]
%   nNeurons: number of neurons
%   maxLagInSeconds: maximum lag between spikes for cross-correlation
%   binSizeInSeconds: 
%   startStopInSeconds: 1x2 vector to indicate time period over which to
%       calculate FC if differs from [0,SimTimeInSeconds]; used for IO
%       analysis

% Output:
%   FC: functional connectivity matrix

function [FC] = functionalConnectivityMatrix(spikeindexes,spiketimes,...
    SimTimeInSeconds,StabilizationTimeInSeconds,nNeurons,...
    maxLagInSeconds,binSizeInSeconds,startStopInSeconds)

if nargin == 7
    edges = 0:binSizeInSeconds:SimTimeInSeconds;
elseif nargin == 8
    edges = startStopInSeconds(1):binSizeInSeconds:startStopInSeconds(2);
elseif nargin > 8
    warning('Too many inputs')
end

%% Format data
spiketimes_afterStart = spiketimes >= StabilizationTimeInSeconds; % omit presimulation time 
spikeindexes = spikeindexes(spiketimes_afterStart) + 1; % add 1 to convert from python indexes to matlab indexes
spiketimes = spiketimes(spiketimes_afterStart) - StabilizationTimeInSeconds;

%% Generate functional connectivity matrix

spiketimesNeuronal = cell(nNeurons,1);
spiketimesNeuronalDiscretized = zeros(nNeurons,length(edges)-1);
FC = ones(nNeurons); % ones because we're not doing the diagonal
for ii = 1:nNeurons
    spiketimesNeuronal{ii} = spiketimes(spikeindexes == ii);
    spiketimesNeuronalDiscretized(ii,:) = histcounts(spiketimesNeuronal{ii},edges);
end

for ii = 1:nNeurons
    spiketimesNeuronal1 = spiketimesNeuronalDiscretized(ii,:);
    parfor jj = (ii+1):nNeurons
        spiketimesNeuronal2 = spiketimesNeuronalDiscretized(jj,:);
        FC(ii,jj) = max(xcorr(spiketimesNeuronal1,spiketimesNeuronal2,...
            maxLagInSeconds/binSizeInSeconds,'coeff'));
    end
end

tril_FC = triu(FC)'; 
FC = tril_FC + triu(FC); % symmetric matrix
